#ifndef SERIALIO_H__
#define SERIALIO_H__

//Fran Jarnjak
//June 2007
// fjarnjak_at_gmail_dot_com

//Sample code of reading and writing to Serial port under Mac
//Formating done for TRI-Plc native protocol (multipoint)

// Tested and works fine under Mac OS X Tiger (10.4.9)
// Uses USB->Serial (RS232) conversion cable from IO-Data which uses PL2303 chipset. 
// (most cables use PL2303 chipset)
// Drivers can be found on Prolific website (makers of the chip) or 
// on sourceforge at:
// http://sourceforge.net/projects/osx-pl2303
// Seems that sourceforge one is newer and works the best. 
//

#include <termios.h>
#include <pthread.h>
#include <unistd.h>
#include <string>

class SerialIO
{
       private:
        SerialIO(const SerialIO& obj);            // Disable copy constructor
        SerialIO& operator=(const SerialIO& obj); // Disable = operator
        
       protected:

        // For Serial port        
        std::string path;
        int baud;
        int dataBits;
        int parity; 
        bool valid;
        
        struct termios termFlags;
        struct termios oldFlags;
        
        int fd;
       
        // opens the serial port 
        void openSerial();
        // closes the serial port
        void closeSerial();
        // thread sleep utility function
        void sleepTime(double fSec);

        // Write port protector mutex
        // Say several threads have a handle on the single SerialIO object. 
        // Make sure serial port can be written to only from one threads. Others will block.
        pthread_mutex_t writeMutex;
                
       public:
        // Constructor
        SerialIO(const char* pth, int br=38400, int db=8, int p=0);
        // Destructor
        ~SerialIO();
        
        // Write functions - returns result
        void sendBytes(const char * data, int wsize, char * rslt, int * rsltSize);
 
        inline bool isValid() const { return valid; }
};

#endif

